<?php
session_start();

// Verifica si el usuario está autenticado
if (!isset($_SESSION['google_user']) && !isset($_SESSION['microsoft_access_token'])) {
    header('Location: login.php');
    exit();
}

header('Content-Type: text/html; charset=utf-8');

// Columnas esperadas en el CSV
$expectedHeaders = [
    "Unique ID", "Request date", "BU", "WBS", "Subject", "Project's supplier", "Project's client",
    "Category", "Speciality", "Mebos ID/Onekey ID", "First name", "Last name", "Street, no. ext., no. int.",
    "Settlement", "City", "State", "ZIP", "Country", "Mobile", "Email", "Bank", "Account", "Remarks", "Reward"
];

// Catálogo de especialidades permitidas
$validSpecialities = [
    "Administrativos (Payers)", "Alergólogo", "Algólogo", "Angiólogo", "Bariatra", "Cardiólogo",
    "Cirujano cardiovascular y torácico", "Cirujano de manos", "Cirujano de transplante", "Cirujano general",
    "Cirujano neurológico", "Cirujano oral", "Cirujano Ortopédico", "Cirujano pediatra", "Cirujano plástico",
    "Cirujano rectal y de colon", "Cirujano traumatólogo", "Cirujano vascular", "Consumidores CH", "Dermatólogo",
    "Endocrinólogo", "Enfermeras", "Expertos de la industria", "Farmacéutico", "FMV especialidad", "Gastroenterólogo",
    "Geriatra", "Ginecólogo", "Hematólogo", "Hepatólogos", "Infectólogo", "Inmunólogo", "Internista", "KOL",
    "Medicina física y del deporte", "Médico cuidados intensivos", "Médico general", "Médico pulmonar/neumólogo",
    "Nefrólogo", "Neumólogo", "Neurólogo", "Nutriólogo", "Odontólogo", "Oftalmólogo", "Oncólogo", "Oncólogo quirúrgico",
    "Ortopedista/ traumaólogo", "Otorrinolaringólogo", "Pacientes", "Pediatra", "Proctólogo", "Psiquiatra",
    "Rehabilitador", "Reumatólogo", "Urgenciólogo", "Urólogo"
];

$error = null;
$csvData = [];
$csvHeaders = null;
$specialityIndex = null;

if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_FILES['csv_file'])) {
    $fileTmpPath = $_FILES['csv_file']['tmp_name'];
    $fileName = $_FILES['csv_file']['name'];
    $fileExtension = pathinfo($fileName, PATHINFO_EXTENSION);

    if ($fileExtension !== 'csv') {
        $error = 'Por favor, sube un archivo CSV válido.';
    } else {
        if (($handle = fopen($fileTmpPath, 'r')) !== false) {
            stream_filter_prepend($handle, "convert.iconv.ISO-8859-1/UTF-8");
            $csvHeaders = fgetcsv($handle);

            if (!$csvHeaders) {
                $error = "No se pudieron leer los encabezados del CSV.";
            } else {
                $missingColumns = array_diff($expectedHeaders, $csvHeaders);
                $extraColumns = array_diff($csvHeaders, $expectedHeaders);

                if (count($missingColumns) > 0) {
                    $error = 'Faltan las siguientes columnas: ' . implode(', ', $missingColumns);
                } elseif (count($extraColumns) > 0) {
                    $error = 'El CSV tiene columnas adicionales: ' . implode(', ', $extraColumns);
                } else {
                    $specialityIndex = array_search("Speciality", $csvHeaders);

                    while (($row = fgetcsv($handle)) !== false) {
                        $row = array_map("utf8_encode", $row);
                        $csvData[] = $row;
                    }
                }
            }
            fclose($handle);

            // Guardar los datos en la sesión para usarlos al enviar las correcciones
            $_SESSION['csv_data'] = $csvData;
            $_SESSION['csv_headers'] = $csvHeaders;
            $_SESSION['speciality_index'] = $specialityIndex;
            $_SESSION['file_name'] = $fileName;
        }
    }
}

// Si el usuario envía las correcciones
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['submit_csv'])) {
    $csvData = $_SESSION['csv_data'];
    $csvHeaders = $_SESSION['csv_headers'];
    $specialityIndex = $_SESSION['speciality_index'];
    $fileName = $_SESSION['file_name'];

    // Actualizar las especialidades corregidas
    foreach ($_POST['speciality'] as $index => $correctedSpeciality) {
        $csvData[$index][$specialityIndex] = $correctedSpeciality;
    }

    // Guardar el archivo corregido temporalmente
    $tempDir = sys_get_temp_dir();
    $correctedFile = $tempDir . '/corrected_' . uniqid() . '.csv';

    $handle = fopen($correctedFile, 'w');
    fputcsv($handle, $csvHeaders);

    foreach ($csvData as $row) {
        fputcsv($handle, $row);
    }
    fclose($handle);

    // Almacenar el nombre del archivo en la sesión para enviarlo a OneDrive
    $_SESSION['corrected_file'] = $correctedFile;

    // Redirigir a OneDrive
    header("Location: onedrive.iqvia.php");
    exit();
}
?>

<!DOCTYPE html>
<html lang="es">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Visualizar y Corregir CSV</title>
    <style>
        .invalid { background-color: #ffcccc; }
        .valid { background-color: #ccffcc; }
    </style>
    <script>
        function updateCell(selectElement) {
            if (selectElement.classList.contains('invalid')) {
                selectElement.classList.remove('invalid');
                selectElement.classList.add('valid');
            }
        }
    </script>
</head>
<body>
    <h2>Visualizar y Corregir CSV</h2>

    <?php if (isset($error)) echo "<p style='color: red;'>$error</p>"; ?>

    <form method="post" enctype="multipart/form-data">
        <input type="file" name="csv_file" required>
        <button type="submit">Subir CSV</button>
    </form>

    <?php if (!empty($_SESSION['csv_data'])): ?>
        <h3>Editar Especialidades Incorrectas:</h3>
        <form method="post">
            <table border="1">
                <thead>
                    <tr>
                        <?php foreach ($_SESSION['csv_headers'] as $header): ?>
                            <th><?= htmlspecialchars($header) ?></th>
                        <?php endforeach; ?>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($_SESSION['csv_data'] as $index => $row): ?>
                        <tr>
                            <?php foreach ($row as $colIndex => $col): ?>
                                <td>
                                    <?php if ($colIndex === $_SESSION['speciality_index']): ?>
                                        <select name="speciality[<?= $index ?>]" onchange="updateCell(this)">
                                            <?php foreach ($validSpecialities as $valid): ?>
                                                <option value="<?= htmlspecialchars($valid) ?>" 
                                                    <?= ($col === $valid) ? 'selected' : '' ?>>
                                                    <?= htmlspecialchars($valid) ?>
                                                </option>
                                            <?php endforeach; ?>
                                        </select>
                                    <?php else: ?>
                                        <?= htmlspecialchars($col) ?>
                                    <?php endif; ?>
                                </td>
                            <?php endforeach; ?>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>
            <button type="submit" name="submit_csv">Subir a OneDrive</button>
        </form>
    <?php endif; ?>
</body>
</html>
