<?php
session_start();
require 'microsoft_config.php'; // Archivo con la configuración (credenciales de Microsoft)

if (!isset($_SESSION['microsoft_access_token'])) {
    header('Location: login.php'); // Si no está autenticado, redirige a login
    exit();
}

$expectedHeaders = [
    "Unique ID", "Request date", "BU", "WBS", "Subject", "Project's supplier", "Project's client",
    "Category", "Speciality", "Mebos ID/Onekey ID", "First name", "Last name", "Street, no. ext., no. int.",
    "Settlement", "City", "State", "ZIP", "Country", "Mobile", "Email", "Bank", "Account", "Remarks", "Reward"
];

$validSpecialities = [
    "Administrativos (Payers)", "Alergólogo", "Algólogo", "Angiólogo", "Bariatra", "Cardiólogo",
    "Cirujano cardiovascular y torácico", "Cirujano de manos", "Cirujano de transplante", "Cirujano general",
    "Cirujano neurológico", "Cirujano oral", "Cirujano Ortopédico", "Cirujano pediatra", "Cirujano plástico",
    "Cirujano rectal y de colon", "Cirujano traumatólogo", "Cirujano vascular", "Consumidores CH", "Dermatólogo",
    "Endocrinólogo", "Enfermeras", "Expertos de la industria", "Farmacéutico", "FMV especialidad", "Gastroenterólogo",
    "Geriatra", "Ginecólogo", "Hematólogo", "Hepatólogos", "Infectólogo", "Inmunólogo", "Internista", "KOL",
    "Medicina física y del deporte", "Médico cuidados intensivos", "Médico general", "Médico pulmonar/neumólogo",
    "Nefrólogo", "Neumólogo", "Neurólogo", "Nutriólogo", "Odontólogo", "Oftalmólogo", "Oncólogo", "Oncólogo quirúrgico",
    "Ortopedista/ traumaólogo", "Otorrinolaringólogo", "Pacientes", "Pediatra", "Proctólogo", "Psiquiatra",
    "Rehabilitador", "Reumatólogo", "Urgenciólogo", "Urólogo"
];

$error = null;
$csvData = [];
$csvHeaders = null;
$specialityIndex = null;

if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_FILES['csv_file'])) {
    $fileTmpPath = $_FILES['csv_file']['tmp_name'];
    $fileName = $_FILES['csv_file']['name'];
    $fileExtension = pathinfo($fileName, PATHINFO_EXTENSION);

    if ($fileExtension !== 'csv') {
        $error = 'Por favor, sube un archivo CSV válido.';
    } else {
        if (($handle = fopen($fileTmpPath, 'r')) !== false) {
            stream_filter_prepend($handle, "convert.iconv.ISO-8859-1/UTF-8");
            $csvHeaders = fgetcsv($handle);

            if (!$csvHeaders) {
                $error = "No se pudieron leer los encabezados del CSV.";
            } else {
                $missingColumns = array_diff($expectedHeaders, $csvHeaders);
                $extraColumns = array_diff($csvHeaders, $expectedHeaders);

                if (count($missingColumns) > 0) {
                    $error = 'Faltan las siguientes columnas: ' . implode(', ', $missingColumns);
                } elseif (count($extraColumns) > 0) {
                    $error = 'El CSV tiene columnas adicionales: ' . implode(', ', $extraColumns);
                } else {
                    $specialityIndex = array_search("Speciality", $csvHeaders);

                    while (($row = fgetcsv($handle)) !== false) {
                        $row = array_map("utf8_encode", $row);
                        $csvData[] = $row;
                    }
                }
            }
            fclose($handle);

            // Guardar los datos en la sesión para usarlos al enviar las correcciones
            $_SESSION['csv_data'] = $csvData;
            $_SESSION['csv_headers'] = $csvHeaders;
            $_SESSION['speciality_index'] = $specialityIndex;
            $_SESSION['file_name'] = $fileName;
        }
    }
}

// Si el usuario envía las correcciones
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['submit_csv'])) {
    $csvData = $_SESSION['csv_data'];
    $csvHeaders = $_SESSION['csv_headers'];
    $specialityIndex = $_SESSION['speciality_index'];
    $fileName = $_SESSION['file_name'];

    // Actualizar las especialidades corregidas
    foreach ($_POST['speciality'] as $index => $correctedSpeciality) {
        $csvData[$index][$specialityIndex] = $correctedSpeciality;
    }

    // Guardar el archivo corregido temporalmente
    $tempDir = sys_get_temp_dir();
    $correctedFile = $tempDir . '/corrected_' . uniqid() . '.csv';

    $handle = fopen($correctedFile, 'w');
    fputcsv($handle, $csvHeaders);

    foreach ($csvData as $row) {
        fputcsv($handle, $row);
    }
    fclose($handle);

    // Almacenar el nombre del archivo en la sesión para enviarlo a OneDrive
    $_SESSION['corrected_file'] = $correctedFile;

    // Preguntar al usuario si desea enviar a OneDrive
    echo "<form method='post' action=''>";
    echo "<p>¿Deseas enviar el archivo corregido a OneDrive?</p>";
    echo "<button type='submit' name='upload_to_onedrive'>Sí, enviar a OneDrive</button>";
    echo "</form>";
}

// Si el usuario decide enviar el archivo a OneDrive
if (isset($_POST['upload_to_onedrive'])) {
    $accessToken = $_SESSION['microsoft_access_token'];
    $correctedFile = $_SESSION['corrected_file'];
    $fileName = $_SESSION['file_name'];
    
    $folderName = 'TestFolder2'; // Asegúrate de que esta carpeta exista
    $url = "https://graph.microsoft.com/v1.0/me/drive/special/approot:/Documents/{$folderName}/{$fileName}:/content";

    // Obtiene la respuesta al intentar subir el archivo
    $response = uploadFileToOneDrive($url, $correctedFile, $accessToken);

    // Verifica el código de respuesta
    if ($response['status_code'] == 200) {
        echo "Archivo subido correctamente. Puedes acceder al archivo en el siguiente enlace: <a href='" . $response['response']['webUrl'] . "' target='_blank'>Acceder al archivo</a>";
    } else {
        echo "Error al subir el archivo. Código HTTP: " . $response['status_code'] . "<br>";
        echo "<pre>" . print_r($response['response'], true) . "</pre>";
    }
}

// Función para enviar el archivo a OneDrive
function uploadFileToOneDrive($url, $fileTmpPath, $accessToken) {
    // Abre el archivo
    $fileContent = file_get_contents($fileTmpPath);

    // Configura los encabezados
    $headers = [
        "Authorization: Bearer {$accessToken}",
        "Content-Type: application/octet-stream", // Tipo de contenido adecuado para archivos binarios
    ];

    // Inicializa cURL
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_UPLOAD, true); // Establece que se hará una subida
    curl_setopt($ch, CURLOPT_INFILE, fopen($fileTmpPath, 'rb')); // Lee el archivo
    curl_setopt($ch, CURLOPT_INFILESIZE, filesize($fileTmpPath)); // Tamaño del archivo
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

    // Ejecuta la petición
    $response = curl_exec($ch);
    $statusCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

    // Cierra cURL
    curl_close($ch);

    // Devuelve la respuesta y el código de estado
    return [
        'status_code' => $statusCode,
        'response' => json_decode($response, true)
    ];
}

?>

<!DOCTYPE html>
<html lang="es">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Subir archivo CSV a OneDrive</title>
</head>
<body>
    <h2>Subir archivo CSV a OneDrive</h2>

    <?php
    if (isset($error)) {
        echo "<p style='color: red;'>$error</p>";
    }

    if (!isset($_SESSION['csv_data'])) {
        echo "<form method='post' enctype='multipart/form-data'>";
        echo "<label for='csv_file'>Selecciona un archivo CSV:</label>";
        echo "<input type='file' name='csv_file' id='csv_file' required>";
        echo "<button type='submit'>Cargar archivo</button>";
        echo "</form>";
    } else {
        echo "<h3>Contenido del CSV:</h3>";
        echo "<form method='post'>";
        echo "<table border='1'><tr>";

        foreach ($_SESSION['csv_headers'] as $header) {
            echo "<th>$header</th>";
        }

        echo "</tr>";

        foreach ($_SESSION['csv_data'] as $index => $row) {
            echo "<tr>";
            foreach ($row as $colIndex => $value) {
                if ($colIndex === $_SESSION['speciality_index']) {
                    echo "<td><select name='speciality[$index]'>";
                    foreach ($validSpecialities as $speciality) {
                        $selected = ($value === $speciality) ? "selected" : "";
                        echo "<option value='$speciality' $selected>$speciality</option>";
                    }
                    echo "</select></td>";
                } else {
                    echo "<td>" . htmlspecialchars($value) . "</td>";
                }
            }
            echo "</tr>";
        }

        echo "</table>";
        echo "<button type='submit' name='submit_csv'>Guardar cambios</button>";
        echo "</form>";
    }
    ?>

</body>
</html>
